#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Selecteer een methode om gegevens te importeren."
wizard_decide_restore="De GitLab-database bestaat al.<br>Selecteer een van de volgende handelingen:"
wizard_restore="Bestaande gegevens gebruiken"
wizard_create_new="Installatie verwijderen (alle bestaande gegevens, inclusief configuratiebestanden en database, worden verwijderd.)"
install_title="Installeer GitLab"
install_data_root_desc="Maak een gedeelde map om de gegevens van GitLab op te slaan."
install_data_root_label="Naam van gedeelde map"
http_port_desc="Voer het HTTP-poortnummer in voor GitLab."
http_port_label="HTTP-poortnummer"
ssh_port_desc="Voer het SSH-poortnummer in voor GitLab."
ssh_port_label="SSH-poortnummer"
hostname_label="Domeinnaam"
admin_email_desc="Voer de GitLab administratore-mailaccount in."
admin_email_label="E-mail"
smtp_enable_desc="Schakel GitLab in om e-mailberichten te verzenden via een SMTP-server."
smtp_enable_label="SMTP inschakelen"
smtp_address_desc="Voer het SMTP-serveradres in van de server die GitLab host."
smtp_address_label="SMTP-adres"
smtp_port_desc="Voer de SMP-serverpoort in."
smtp_port_label="SMTP-port"
smtp_user_desc="Voer de SMP-servergebruikersnaam in."
smtp_user_label="SMTP-gebruikersnaam"
smtp_pass_desc="Voer het SMP-serverwachtwoord in."
smtp_pass_label="SMTP-wachtwoord"
smtp_verify_desc="Bij verbinding met GitLab verifieert de SMTP-server het SSL-certificaat."
smtp_verify_label="Verifieer het SSL-certificaat en weiger onveilige verbindingen."
default_gitlab_account_desc="De standaard aanmeldingsgegevens zijn root/5iveL!fe"
additional_memory_required="Onvoldoende geheugen"
recommended_memory="Aanbevolen geheugengrootte"
db_connect_error="De verbinding met GitLab is mislukt. Vergewis u ervan dat de database correct werkt."
mem_total_check_fail="Minstens {1} RAM is vereist om <a href=\"{2}\" target=\"_blank\">te installeren of bij te werken tot de laatste versie van GitLab</a>. Breid het geheugen van uw Synology NAS uit en probeer het opnieuw."
mem_total_check_warning="Minstens {1} RAM is vereist om GitLab te installeren of bij te werken. Onvoldoende geheugen kan onverwachte fouten veroorzaken wanneer u GitLab uitvoert. Daarom raden we u aan om het geheugen van uw Synology NAS uit te breiden. <a href=\"{2}\" target=\"_blank\">Meer weten</a>."
other_used_ports_warning="Opmerking: dit pakket gebruikt ook de poorten {1} en {2} op de Docker-interface. Open deze poorten voor alle interfaces in de firewall."
domain_value_warning="De gespecificeerde domeinnaam wordt ook gebruikt als het adres van GitLab in e-mails. We raden u aan om een andere naam dan "localhost" of "127.0.0.1" op te geven wanneer u een domeinnaam hebt voor GitLab."
upgrade_warning_page_title="Docker-GitLab upgraden"
postgresql_warning_page_desc="De nieuwe versie van Docker-GitLab vervangt MariaDB met <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> als back-enddatabase."
upgrade_agree="Ik ga akkoord met de upgrade en conversie van de database naar PostgreSQL."
intermediate_migrate_warning_page_desc="De huidige versie is verouderd voor het bijwerken naar de nieuwste versie. Als u wilt updaten naar de nieuwste versie, annuleert u eerst het huidige updateproces, downloadt u <a href=\"{1}\" target=\"_blank\">hier</a> een tussenversie, update u handmatig naar die versie en vervolgens naar de nieuwste versie. Voor meer informatie over handmatig bijwerken, raadpleeg <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">dit artikel</a>."
PageRestore()
{
	local is_import_valid="true"
	[ ! -z "$1" ] && local warning_page_desc="$1"
	[ ! -z "$2" ] && is_import_valid="false"
cat << EOF
{
	"step_title": "$wizard_found_backup",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "singleselect",
		"desc": "$wizard_decide_restore",
		"subitems": [{
			"key": "restore_backup",
			"desc": "${wizard_restore}${warning_page_desc}",
			"defaultValue": true,
			"validator": {
				"fn": "{return arguments[0] ? ${is_import_valid} : true;}"
			}
		}, {
			"desc": "$wizard_create_new",
			"defaultValue": false
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "worker_mode",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}]
}
EOF
}

PageRestorePostgresql()
{
	PageRestore "<br>$(echo "$postgresql_warning_page_desc" | quote_json)"
}

PageRestoreIntermediateMigrate()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
	PageRestore "<br><b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>" "false"
}
//in DSM 6.x
CheckRestoreV1()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.items.items[i];
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

//Wizard add step container in DSM 7.0.
CheckRestoreV2()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.ownerCt.items.items[i].getStep();
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

CheckRestore()
{
	dsm_build_num=$(/bin/uname -v | cut -d' ' -f1 | sed -e "s|#||g")
	if [ "$dsm_build_num" -ge 40000 ]; then
		CheckRestoreV2
	else
		CheckRestoreV1
	fi
}

FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" === item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		restore = $DEFAULT_RESTORE; // user restore or not, set by is_restore
		// will set restore
		$(CheckRestore)

		if (arguments[0].items.items[0].value === \"${install_data_root_desc}\") {
			dataroot_textfield = null;
			$(FindObj "pkgwizard_dataroot" "dataroot_textfield")
			if (restore) {
				dataroot_textfield.setValue(\"@OLD_DATAROOT@\");
			}
			dataroot_textfield.setDisabled(restore);
		}
	}",
EOF
}

ApplyDataRootInfo()
{
	local page="$1"
	sed "s/@OLD_DATAROOT@/$OLD_DATAROOT/g" <<< "$page"
}

PageInstallSetting()
{
	other_used_ports_warning="<br><b>$other_used_ports_warning</b>"
	other_used_ports_warning=${other_used_ports_warning//\{1\}/5432}
	other_used_ports_warning=${other_used_ports_warning//\{2\}/6379}
	domain_value_warning="<br><b>$domain_value_warning</b>"

	local page=$(cat << EOF
{
	"step_title": "$install_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$install_data_root_desc",
		"subitems": [{
			"key": "pkgwizard_dataroot",
			"desc": "$install_data_root_label",
			"defaultValue": "gitlab",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[^/]+$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$http_port_desc",
		"subitems": [{
			"key": "pkgwizard_http_port",
			"desc": "$http_port_label",
			"defaultValue": "30000",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$ssh_port_desc",
		"subitems": [{
			"key": "pkgwizard_ssh_port",
			"desc": "$ssh_port_label",
			"defaultValue": "30001",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	}, {
		"desc": "$other_used_ports_warning"
	}]
},{
	"step_title": "$install_title",
	"items": [{
		"type": "textfield",
		"desc": "$hostname_desc",
		"subitems": [{
			"key": "pkgwizard_hostname",
			"desc": "$hostname_label",
			"defaultValue": "localhost",
			"validator": {
				"allowBlank": false
			}
		}]
	},{
		"type": "textfield",
		"desc": "$admin_email_desc",
		"subitems": [{
			"key": "pkgwizard_admin_email",
			"desc": "$admin_email_label",
			"defaultValue": "example@example.com",
			"validator": {
				"allowBlank": false,
				"vtype": "email"
			}
		}]
	}, {
		"desc": "$domain_value_warning"
	}]
},{
	"step_title": "$install_title",
	$(Activate)
	"items": [{
		"type": "multiselect",
		"desc": "$smtp_enable_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_enable",
			"desc": "$smtp_enable_label",
			"defaultValue": false
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_address_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_address",
			"desc": "$smtp_address_label"
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_port_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_port",
			"desc": "$smtp_port_label",
			"defaultValue": "587",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_user_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_user",
			"desc": "$smtp_user_label"
		}]
	},{
		"type": "password",
		"desc": "$smtp_pass_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_pass",
			"desc": "$smtp_pass_label"
		}]
	},{
		"type": "multiselect",
		"desc": "$smtp_verify_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_verify",
			"desc": "$smtp_verify_label",
			"defaultValue": true
		}]
	}]
}
EOF
)
	ApplyDataRootInfo "$page"
}
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
main()
{
	local install_page=""
	local memory_check_page=""
	local restore_page=""
	local install_setting_page=""

	DEFAULT_RESTORE=false
	OLD_DATAROOT="$(get_share)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_restore; then
		. "$BACKUP_CONFIG"
		if need_upgrade_to_intermediate_version_first; then
			restore_page="$(PageRestoreIntermediateMigrate)"
		elif [ ! -z $VERSION ] && need_convert_db "$VERSION"; then
			restore_page="$(PageRestorePostgresql)"
		else
			restore_page="$(PageRestore)"
		fi
	fi
	install_setting_page="$(PageInstallSetting)"

	install_page=$(page_append "$install_page" "$memory_check_page")
	install_page=$(page_append "$install_page" "$restore_page")
	install_page=$(page_append "$install_page" "$install_setting_page")
	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
