#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Selecione um método parar importar os dados."
wizard_decide_restore="GitLab o bando de dados já existe.<br>Selecione uma das seguintes ações:"
wizard_restore="Usar dados existentes"
wizard_create_new="Limpar instalação (todos os dados existentes, incluindo arquivos de configuração e o banco de dados, serão removidos.)"
install_title="Instale GitLab"
install_data_root_desc="Crie uma pasta compartilhada para armazenar os dados de GitLab."
install_data_root_label="Nome de pasta compartilhada"
http_port_desc="Digite o número da porta HTTP para GitLab."
http_port_label="Número da porta HTTP"
ssh_port_desc="Digite o número da porta SSH para GitLab."
ssh_port_label="Número de porta SSH"
hostname_label="Nome de domínio"
admin_email_desc="Digite a conta de e-mail do administrador GitLab."
admin_email_label="e-mail"
smtp_enable_desc="Habilite GitLab para enviar e-mails através de um servidor SMTP."
smtp_enable_label="Ativar <span>SMTP</span>"
smtp_address_desc="Digite a hospedagem do endereço do servidor SMTP GitLab."
smtp_address_label="Endereço SMTP"
smtp_port_desc="Digite a porta do servidor SMTP."
smtp_port_label="porta SMTP"
smtp_user_desc="Digite o nome de usuário do servidor SMTP."
smtp_user_label="Nome de usuário SMTP"
smtp_pass_desc="Digite a senha do servidor SMTP."
smtp_pass_label="Senha SMTP"
smtp_verify_desc="Quando GitLab estiver conectado, verifique o certificado SSL do servidor SMTP."
smtp_verify_label="Verifique o certificado SSL e recuse as conexões inseguras."
default_gitlab_account_desc="A credencial de login padrão é root/5iveL!fe"
additional_memory_required="Memória insuficiente"
recommended_memory="Tamanho recomendado da memória"
db_connect_error="Falha ao se conectar a GitLab. Certifique-se de que o banco de dados esteja executando adequadamente."
mem_total_check_fail="Um mínimo de {1} RAM é necessário para instalar ou atualizar <a href=\"{2}\" target=\"_blank\">a versão mais recente do GitLab</a>. Expanda a memória do Synology NAS e tente novamente."
mem_total_check_warning="Um mínimo de {1} RAM é necessário para instalar ou atualizar o GitLab. Memória insuficiente pode causar erros inesperados quando você executar o GitLab; assim, recomendamos expandir a memória do Synology NAS. <a href=\"{2}\" target=\"_blank\">Saiba mais</a>."
other_used_ports_warning="Observação: Este pacote também usa as portas {1} e {2} na interface do Docker. Permita essas portas para todas as interfaces no Firewall."
domain_value_warning="O nome de domínio especificado também será usado como o endereço do GitLab nos e-mails. Recomendamos especificar um nome diferente de "localhost" ou "127.0.0.1" quando você tiver um nome de domínio para o GitLab."
upgrade_warning_page_title="Atualizar o Docker-GitLab"
postgresql_warning_page_desc="A nova versão do Docker-GitLab substituirá o MariaDB por <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> como banco de dados de backend."
upgrade_agree="Concordo em atualizar e converter o banco de dados em PostgreSQL."
intermediate_migrate_warning_page_desc="A versão atual está desatualizada para a atualização para a versão mais recente. Para atualizar para a versão mais recente, primeiro cancele o processo de atualização atual, baixe uma versão intermediária <a href=\"{1}\" target=\"_blank\">aqui</a>, atualize manualmente para essa versão e, em seguida, atualize para a versão mais recente. Para obter mais informações sobre a atualização manual, consulte <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">este artigo</a>."
PageRestore()
{
	local is_import_valid="true"
	[ ! -z "$1" ] && local warning_page_desc="$1"
	[ ! -z "$2" ] && is_import_valid="false"
cat << EOF
{
	"step_title": "$wizard_found_backup",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "singleselect",
		"desc": "$wizard_decide_restore",
		"subitems": [{
			"key": "restore_backup",
			"desc": "${wizard_restore}${warning_page_desc}",
			"defaultValue": true,
			"validator": {
				"fn": "{return arguments[0] ? ${is_import_valid} : true;}"
			}
		}, {
			"desc": "$wizard_create_new",
			"defaultValue": false
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "worker_mode",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}]
}
EOF
}

PageRestorePostgresql()
{
	PageRestore "<br>$(echo "$postgresql_warning_page_desc" | quote_json)"
}

PageRestoreIntermediateMigrate()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
	PageRestore "<br><b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>" "false"
}
//in DSM 6.x
CheckRestoreV1()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.items.items[i];
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

//Wizard add step container in DSM 7.0.
CheckRestoreV2()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.ownerCt.items.items[i].getStep();
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

CheckRestore()
{
	dsm_build_num=$(/bin/uname -v | cut -d' ' -f1 | sed -e "s|#||g")
	if [ "$dsm_build_num" -ge 40000 ]; then
		CheckRestoreV2
	else
		CheckRestoreV1
	fi
}

FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" === item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		restore = $DEFAULT_RESTORE; // user restore or not, set by is_restore
		// will set restore
		$(CheckRestore)

		if (arguments[0].items.items[0].value === \"${install_data_root_desc}\") {
			dataroot_textfield = null;
			$(FindObj "pkgwizard_dataroot" "dataroot_textfield")
			if (restore) {
				dataroot_textfield.setValue(\"@OLD_DATAROOT@\");
			}
			dataroot_textfield.setDisabled(restore);
		}
	}",
EOF
}

ApplyDataRootInfo()
{
	local page="$1"
	sed "s/@OLD_DATAROOT@/$OLD_DATAROOT/g" <<< "$page"
}

PageInstallSetting()
{
	other_used_ports_warning="<br><b>$other_used_ports_warning</b>"
	other_used_ports_warning=${other_used_ports_warning//\{1\}/5432}
	other_used_ports_warning=${other_used_ports_warning//\{2\}/6379}
	domain_value_warning="<br><b>$domain_value_warning</b>"

	local page=$(cat << EOF
{
	"step_title": "$install_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$install_data_root_desc",
		"subitems": [{
			"key": "pkgwizard_dataroot",
			"desc": "$install_data_root_label",
			"defaultValue": "gitlab",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[^/]+$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$http_port_desc",
		"subitems": [{
			"key": "pkgwizard_http_port",
			"desc": "$http_port_label",
			"defaultValue": "30000",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$ssh_port_desc",
		"subitems": [{
			"key": "pkgwizard_ssh_port",
			"desc": "$ssh_port_label",
			"defaultValue": "30001",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	}, {
		"desc": "$other_used_ports_warning"
	}]
},{
	"step_title": "$install_title",
	"items": [{
		"type": "textfield",
		"desc": "$hostname_desc",
		"subitems": [{
			"key": "pkgwizard_hostname",
			"desc": "$hostname_label",
			"defaultValue": "localhost",
			"validator": {
				"allowBlank": false
			}
		}]
	},{
		"type": "textfield",
		"desc": "$admin_email_desc",
		"subitems": [{
			"key": "pkgwizard_admin_email",
			"desc": "$admin_email_label",
			"defaultValue": "example@example.com",
			"validator": {
				"allowBlank": false,
				"vtype": "email"
			}
		}]
	}, {
		"desc": "$domain_value_warning"
	}]
},{
	"step_title": "$install_title",
	$(Activate)
	"items": [{
		"type": "multiselect",
		"desc": "$smtp_enable_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_enable",
			"desc": "$smtp_enable_label",
			"defaultValue": false
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_address_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_address",
			"desc": "$smtp_address_label"
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_port_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_port",
			"desc": "$smtp_port_label",
			"defaultValue": "587",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_user_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_user",
			"desc": "$smtp_user_label"
		}]
	},{
		"type": "password",
		"desc": "$smtp_pass_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_pass",
			"desc": "$smtp_pass_label"
		}]
	},{
		"type": "multiselect",
		"desc": "$smtp_verify_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_verify",
			"desc": "$smtp_verify_label",
			"defaultValue": true
		}]
	}]
}
EOF
)
	ApplyDataRootInfo "$page"
}
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
main()
{
	local install_page=""
	local memory_check_page=""
	local restore_page=""
	local install_setting_page=""

	DEFAULT_RESTORE=false
	OLD_DATAROOT="$(get_share)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_restore; then
		. "$BACKUP_CONFIG"
		if need_upgrade_to_intermediate_version_first; then
			restore_page="$(PageRestoreIntermediateMigrate)"
		elif [ ! -z $VERSION ] && need_convert_db "$VERSION"; then
			restore_page="$(PageRestorePostgresql)"
		else
			restore_page="$(PageRestore)"
		fi
	fi
	install_setting_page="$(PageInstallSetting)"

	install_page=$(page_append "$install_page" "$memory_check_page")
	install_page=$(page_append "$install_page" "$restore_page")
	install_page=$(page_append "$install_page" "$install_setting_page")
	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
