#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="请选择数据导入方式。"
wizard_decide_restore="GitLab 数据库已存在。<br>请选择下列操作之一："
wizard_restore="使用现有数据"
wizard_create_new="全新安装（包括配置文件及数据库的原有数据将全部移除）。"
install_title="安装 GitLab"
install_data_root_desc="创建共享文件夹以存储 GitLab 的数据。"
install_data_root_label="共享文件夹名称"
http_port_desc="请为 GitLab 输入 HTTP 端口号。"
http_port_label="HTTP 端口号"
ssh_port_desc="请为 GitLab 输入 SSH 端口号。"
ssh_port_label="SSH 端口号"
hostname_label="域名"
admin_email_desc="请输入 GitLab 管理员电子邮件帐户。"
admin_email_label="电子邮件"
smtp_enable_desc="启用 GitLab 可通过 SMTP 服务器发送电子邮件。"
smtp_enable_label="启用 SMTP"
smtp_address_desc="请输入 SMTP 服务器 IP 地址主机 GitLab。"
smtp_address_label="SMTP 地址"
smtp_port_desc="请输入 SMTP 服务器端口。"
smtp_port_label="SMTP 端口"
smtp_user_desc="请输入 SMTP 服务器密名称。"
smtp_user_label="SMTP 用户名"
smtp_pass_desc="请输入 SMTP 服务器密码。"
smtp_pass_label="SMTP 密码"
smtp_verify_desc="连接 GitLab 后，验证 SMTP 服务器的 SSL 证书。"
smtp_verify_label="验证 SSL 证书并拒绝不安全的连接。"
default_gitlab_account_desc="默认登录证书为 root/5iveL!fe"
additional_memory_required="内存不足"
recommended_memory="建议内存容量"
db_connect_error="无法连接到 GitLab。请确认数据库正常运行。"
mem_total_check_fail="安装或更新<a href=\"{2}\" target=\"_blank\">最新版本的 GitLab</a> 至少需要 {1} RAM。请扩充 Synology NAS 的内存，然后再试一次。"
mem_total_check_warning="安装或更新 GitLab 至少需要 {1} RAM。内存不足可能会在运行 GitLab 时导致意外错误，因此建议扩充 Synology NAS 的内存。<a href=\"{2}\" target=\"_blank\">更多信息</a>。"
other_used_ports_warning="注：此套件也在 Docker 接口上使用端口 {1}和{2}。请允许防火墙的所有接口访问这些端口。"
domain_value_warning="指定的域名还将在电子邮件中用作 GitLab 地址。如果您拥有 GitLab 域名，则建议指定名称，而不使用“localhost”或“127.0.0.1”。"
upgrade_warning_page_title="升级 Docker-GitLab"
postgresql_warning_page_desc="新版本的 Docker-GitLab 将使用 <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> 替代 MariaDB 作为后端数据库。"
upgrade_agree="我同意升级并将数据库转换为 PostgreSQL。"
intermediate_migrate_warning_page_desc="目前版本已过时，无法更新至最新版本。请先取消目前更新程序，在<a href=\"{1}\" target=\"_blank\">此处</a>下载任一中间版本并进行手动更新，再更新至最新版本。有关手动更新的更多信息，请参阅<a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">本文</a>。"
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
