#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Vyberte způsob importu dat."
wizard_decide_restore="GitLab databáze již existuje.<br>Vyberte některou z následujících akcí:"
wizard_restore="Použít stávající data"
wizard_create_new="Čistá instalace (Odstraní se všechna stávající data včetně konfiguračních souborů a databáze.)"
install_title="Instalovat GitLab"
install_data_root_desc="Vytvořit sdílenou složku pro uložení dat GitLab."
install_data_root_label="Název sdílené složky"
http_port_desc="Zadejte číslo portu HTTP pro GitLab."
http_port_label="Číslo portu HTTP"
ssh_port_desc="Zadejte číslo portu SSH pro GitLab."
ssh_port_label="Číslo portu SSH"
hostname_label="Název domény"
admin_email_desc="Zadejte e-mailový účet správce GitLab."
admin_email_label="e-mail"
smtp_enable_desc="Po povolení možnosti GitLab se budou e-maily odesílat pomocí serveru SMTP."
smtp_enable_label="Povolit SMTP"
smtp_address_desc="Zadejte adresu serveru SMTP hostujícího GitLab."
smtp_address_label="Adresa SMTP"
smtp_port_desc="Zadejte port serveru SMTP."
smtp_port_label="Port SMTP"
smtp_user_desc="Zadejte uživatelské jméno serveru SMTP."
smtp_user_label="Uživatelské jméno SMTP"
smtp_pass_desc="Zadejte heslo serveru SMTP."
smtp_pass_label="Heslo SMTP"
smtp_verify_desc="Po připojení GitLab ověřte certifikát SSL serveru SMTP."
smtp_verify_label="Ověřte certifikát SSL a odmítněte nebezpečná připojení."
default_gitlab_account_desc="Výchozí přihlašovací ověření je root/5iveL!fe"
additional_memory_required="Nedostatek paměti"
recommended_memory="Doporučená velikost paměti"
db_connect_error="Připojení k databázi GitLab se nezdařilo. Přesvědčte se, jestli je databáze správně spuštěná."
mem_total_check_fail="K instalaci nebo aktualizaci <a href=\"{2}\" target=\"_blank\">nejnovější verze databáze GitLab</a> je vyžadováno minimálně {1} paměti RAM. Rozšiřte paměť zařízení Synology NAS a zkuste to znovu."
mem_total_check_warning="K instalaci nebo aktualizaci databáze GitLab je vyžadováno minimálně {1} paměti RAM. Nedostatek paměti může při spuštění databáze GitLab způsobit neočekávané chyby, proto doporučujeme rozšířit paměť zařízení Synology NAS. <a href=\"{2}\" target=\"_blank\">Další informace</a>."
other_used_ports_warning="Poznámka: Tento balíček také používá porty {1} a {2} na rozhraní Dockeru. Povolte tyto porty v bráně firewall pro všechna rozhraní."
domain_value_warning="Zadaný název domény se bude v e-mailech také používat jako adresa GitLab. Pokud máte název domény pro GitLab, doporučujeme zadat jiný název než „localhost“ nebo „127.0.0.1“."
upgrade_warning_page_title="Upgradovat Docker-GitLab"
postgresql_warning_page_desc="Nová verze Docker-GitLab nahradí databázi MariaDB databází backend <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a>."
upgrade_agree="Souhlasím s upgradováním a převedením databáze na PostgreSQL."
intermediate_migrate_warning_page_desc="Současná verze je pro aktualizaci na nejnovější verzi zastaralá. Jestliže chcete aktualizovat na nejnovější verzi, zrušte nejprve současný proces aktualizace, <a href=\"{1}\" target=\"_blank\">pod tímto odkazem</a> si stáhněte přechodnou verzi, ručně aktualizujte na tuto verzi a poté aktualizujte na nejnovější verzi. Další informace o ruční aktualizace se nacházejí v <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">tomto článku</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
