#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Vælg en metode til at importere data."
wizard_decide_restore="GitLab-databasen findes allerede.<br>Vælg en af følgende handlinger:"
wizard_restore="Brug eksisterende data"
wizard_create_new="Ren installation (alle eksisterende data, herunder konfigurationsfiler og databasen bliver fjernet)."
install_title="Installer GitLab"
install_data_root_desc="Opret en delt mappe for at lagre data for GitLab."
install_data_root_label="Navn på delt mappe"
http_port_desc="Indtast HTTP-portnummer for GitLab."
http_port_label="HTTP-portnummer"
ssh_port_desc="Indtast SSH-portnummer for GitLab."
ssh_port_label="SSH-portnummer"
hostname_label="Domænenavn"
admin_email_desc="Indtast GitLab administratorens e-mailkonto."
admin_email_label="e-mail"
smtp_enable_desc="Aktiver GitLab for at sende e-mails via en SMTP-server."
smtp_enable_label="Aktiver SMTP"
smtp_address_desc="Indtast SMTP-serveradressen, der er vært for GitLab."
smtp_address_label="SMTP-adresse"
smtp_port_desc="Indtast SMTP-serverporten."
smtp_port_label="SMTP-port"
smtp_user_desc="Indtast SMTP-serverens brugernavn."
smtp_user_label="SMTP-brugernavn"
smtp_pass_desc="Indtast SMTP-serverens adgangskode."
smtp_pass_label="SMTP-adgangskode"
smtp_verify_desc="Når GitLab er tilsluttet, skal du bekræfte SMTP-serverens SSL-certifikat."
smtp_verify_label="Bekræft SSL-certifikatet, og afvis usikre forbindelser."
default_gitlab_account_desc="Standard loginoplysninger er root/5iveL!fe"
additional_memory_required="Ikke nok hukommelse"
recommended_memory="Anbefalet hukommelsesstørrelse"
db_connect_error="Kunne ikke oprette forbindelse til GitLab. Sørg for, at databasen kører korrekt."
mem_total_check_fail="Et min. på {1} RAM er påkrævet til installation eller opdatering af <a href=\"{2}\" target=\"_blank\">den seneste version af GitLab</a>. Udvid hukommelsen på din Synology NAS, og prøv igen."
mem_total_check_warning="Et min. på {1} RAM er påkrævet til installation eller opdatering af GitLab. Utilstrækkelig hukommelse kan medføre uventede fejl, når du kører GitLab, så det anbefales, at du udvier hukommelsen på din Synology NAS. <a href=\"{2}\" target=\"_blank\">Få mere at vide</a>."
other_used_ports_warning="Bemærk: Denne pakke bruger også portene {1} og {2} på Docker-interface. Tillad disse porte for alle interfaces i firewall."
domain_value_warning="Det angivne domænenavn vil også blive brugt som adressen for GitLab i e-mails. Vi anbefaler, at du angiver et andet navn end "localhost" eller "127.0.0.1", når du har et domænenavn til GitLab."
upgrade_warning_page_title="Opgrader Docker-GitLab"
postgresql_warning_page_desc="Den nye version af Docker-GitLab vil erstatte MariaDB med <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> som backend-database."
upgrade_agree="Jeg accepterer at opgradere og konvertere til PostgreSQL."
intermediate_migrate_warning_page_desc="Den aktuelle version er forældet og opdateres ikke til den nyeste version. Hvis du vil opdatere til den nyeste version, skal du først annullere den aktuelle opdateringsproces, downloade en midlertidig version <a href=\"{1}\" target=\"_blank\">herfra</a>, manuelt opdatere til denne version og derefter opdatere til den nyeste version. Du kan finde flere oplysninger om manuel opdatering i <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">denne artikel</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
