#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Válasszon egy módszert az adatok importálásához."
wizard_decide_restore="GitLab adatbázis már létezik.<br>Válasszon az alábbi műveletek közül:"
wizard_restore="Meglévő adatok használata"
wizard_create_new="Tiszta telepítés (A rendszer minden meglévő adatot, ideértve a konfigurációs fájlokat és az adatbázist eltávolít.)"
install_title="GitLab telepítése"
install_data_root_desc="Hozzon létre megosztott mappát a(z) GitLab adatainak tárolásához."
install_data_root_label="Megosztott mappa neve"
http_port_desc="Kérjük, adja meg a(z) GitLab HTTP port számát."
http_port_label="HTTP portszám"
ssh_port_desc="Kérjük, adja meg a(z) GitLab SSH port számát."
ssh_port_label="SSH portszám"
hostname_label="Tartománynév"
admin_email_desc="Adja meg a GitLab rendszergazda e-mail fiókját."
admin_email_label="e-mail"
smtp_enable_desc="Engedélyezze a következőt: GitLab, amennyiben SMTP-kiszolgálón keresztül szeretne e-maileket küldeni."
smtp_enable_label="SMTP engedélyezése"
smtp_address_desc="Adja meg a következő gazdájaként szolgáló SMTP-kiszolgáló címét: GitLab."
smtp_address_label="SMTP-cím"
smtp_port_desc="Adja meg az SMTP-kiszolgáló portját."
smtp_port_label="SMTP-port"
smtp_user_desc="Adja meg az SMTP-kiszolgáló felhasználónevét."
smtp_user_label="SMTP-felhasználónév"
smtp_pass_desc="Adja meg az SMTP-kiszolgáló jelszavát."
smtp_pass_label="SMTP-jelszó"
smtp_verify_desc="A(z) GitLab csatlakoztatása után ellenőrizze az SMTP-kiszolgáló SSL-tanúsítványát."
smtp_verify_label="Ellenőrizze az SSL-tanúsítványt és utasítsa el a nem biztonságos kapcsolatokat."
default_gitlab_account_desc="Az alapértelmezett bejelentkezési azonosító: root/5iveL!fe"
additional_memory_required="Nincs elegendő memória"
recommended_memory="Ajánlott memóriaméret"
db_connect_error="Nem sikerült csatlakozni a következőhöz: GitLab. Ellenőrizze, hogy az adatbázis megfelelően fut-e."
mem_total_check_fail="A <a href=\"{2}\" target=\"_blank\">GitLab legújabb verziójának</a> telepítéséhez és frissítéséhez legalább {1} RAM szükséges. Bővítse a Synology NAS memóriáját, majd próbálja újra."
mem_total_check_warning="A GitLab telepítéséhez és frissítéséhez legalább {1} RAM szükséges. Ha nincs elegendő memória, a GitLab használata során váratlan hibák keletkezhetnek, ezért javasoljuk, hogy bővítse a Synology NAS memóriáját. <a href=\"{2}\" target=\"_blank\">További tudnivalók</a>."
other_used_ports_warning="Megjegyzés: Ez a csomag is használja a következő portokat a Docker-interfészen: {1} és {2}. Engedélyezze ezeket a portokat az összes interfésznél a tűzfal beállításaiban."
domain_value_warning="A meghatározott tartománynév a GitLab címeként fog szerepelni az e-mailekben. Javasoljuk, hogy egy, a „localhost” és az „127.0.0.1” elnevezéstől eltérő tartománynevet adjon meg a GitLabhez."
upgrade_warning_page_title="Docker-GitLab frissítése"
postgresql_warning_page_desc="A Docker-GitLab új verziója a <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> rendszert fogja belső adatbázisként használni a MariaDB helyett."
upgrade_agree="Engedélyezem, hogy a rendszer az adatbázist a PostgreSQL rendszerre frissítse és konvertálja."
intermediate_migrate_warning_page_desc="Az aktuális verzió elavult, ezért a legújabb verzióra való frissítés nem lehetséges. A legújabb verzióra való frissítéshez először szakítsa meg az aktuális frissítse folyamatot, töltse le a közbenső verziót <a href=\"{1}\" target=\"_blank\">innen</a>, manuálisan frissítsen erre a verzióra, majd frissítsen a legújabb verzióra. A manuális frissítéssel kapcsolatos további információkért olvassa el <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">ezt a cikket</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
