#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Selezionare un metodo per importare i dati."
wizard_decide_restore="Il database GitLab esiste già.<br>Selezionare una delle seguenti azioni:"
wizard_restore="Usare i dati esistenti"
wizard_create_new="Cancellare l'installazione (tutti i dati esistenti, inclusi i file di configurazione e il database, saranno rimossi)."
install_title="Installare GitLab"
install_data_root_desc="Creare una cartella condivisa per archiviare i dati di GitLab."
install_data_root_label="Nome cartella condivisa"
http_port_desc="Inserire il numero di porta HTTP per GitLab."
http_port_label="Numero di prota HTTP"
ssh_port_desc="Inserire il numero di porta SSH per GitLab."
ssh_port_label="Numero porta SSH"
hostname_label="Nome dominio"
admin_email_desc="Inserire l'account e-mail dell'amministratore GitLab."
admin_email_label="e-mail"
smtp_enable_desc="Abilitare GitLab per inviare e-mail tramite un server SMTP."
smtp_enable_label="Abilitare SMTP"
smtp_address_desc="Inserire l'indirizzo del server SMTP che ospita GitLab."
smtp_address_label="Indirizzo SMTP"
smtp_port_desc="Immettere la porta del server SMTP."
smtp_port_label="Porta SMTP"
smtp_user_desc="Immettere il nome utente del server SMTP."
smtp_user_label="Nome utente SMTP"
smtp_pass_desc="Immettere la password del server SMTP."
smtp_pass_label="Password SMTP"
smtp_verify_desc="Quando GitLab è connesso, verificare il certificato SSL del server SMTP."
smtp_verify_label="Verificare il certificato SSL e rifiutare connessioni non sicure."
default_gitlab_account_desc="La creadenziale di accesso predefinita è root/5iveL!fe"
additional_memory_required="Memoria insufficiente"
recommended_memory="Dimensioni di memoria consigliate"
db_connect_error="Connessione a GitLab non riuscita. Verificare che il database sia eseguito correttamente."
mem_total_check_fail="È richiesto un minimo di {1} RAM per installare o aggiornare <a href=\"{2}\" target=\"_blank\">la versione più recente di GitLab</a>. Espandere la memoria di Synology NAS e riprovare."
mem_total_check_warning="È richiesto un minimo di {1} RAM per installare o aggiornare GitLab. Una memoria insufficiente potrebbe causare errori imprevisti durante l'esecuzione di GitLab, quindi si consiglia di espandere la memoria del Synology NAS. <a href=\"{2}\" target=\"_blank\">Per saperne di più</a>."
other_used_ports_warning="Nota: Questo pacchetto utilizza anche le porte {1} e {2} sull'interfaccia Docker. Consentire queste porte per tutte le interfacce nel firewall."
domain_value_warning="Il nome dominio specificato sarà usato come indirizzo di GitLab nelle e-mail. Si consiglia di specificare un nome diverso da "localhost" o "127.0.0.1" quando si dispone di un nome dominio per GitLab."
upgrade_warning_page_title="Esegui upgrade di Docker-GitLab"
postgresql_warning_page_desc="La nuova versione di Docker-GitLab sostituirà MariaDB con <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> come database di backend."
upgrade_agree="Accetto di aggiornare e convertire il database a PostgreSQL."
intermediate_migrate_warning_page_desc="La versione corrente è obsoleta per l'aggiornamento alla versione più recente. Per aggiornare alla versione più recente, per prima cosa, annullare il processo di aggiornamento corrente, scaricare una versione intermedia <a href=\"{1}\" target=\"_blank\">qui</a>, eseguire l'aggiornamento manuale a tale versione, quindi aggiornare alla versione più recente. Per ulteriori informazioni sull'aggiornamento manuale, consultare <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">questo articolo</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
