#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="데이터를 가져올 방법을 선택하십시오."
wizard_decide_restore="GitLab 데이터베이스가 이미 있습니다.<br>다음 작업 중 하나를 선택하십시오."
wizard_restore="기존 데이터 사용"
wizard_create_new="설치 정리(구성 파일 및 데이터베이스를 포함한 모든 기존 데이터가 제거됩니다.)"
install_title="[0] 설치"
install_data_root_desc="GitLab의 데이터를 저장할 공유 폴더를 만듭니다."
install_data_root_label="공유 폴더 이름"
http_port_desc="GitLab의 HTTP 포트 번호를 입력하십시오."
http_port_label="HTTP 포트 번호"
ssh_port_desc="GitLab의 SSH 포트 번호를 입력하십시오."
ssh_port_label="SSH 포트 번호."
hostname_label="도메인 이름"
admin_email_desc="GitLab 관리자 이메일 계정을 입력하십시오."
admin_email_label="이메일"
smtp_enable_desc="GitLab을(를) 활성화하여 SMTP 서버를 통해 이메일을 보냅니다."
smtp_enable_label="SMTP 활성화"
smtp_address_desc="GitLab을(를) 호스팅하는 SMTP  서버 주소를 입력하십시오."
smtp_address_label="SMTP 주소"
smtp_port_desc="SMTP 서버 포트를 입력하십시오."
smtp_port_label="SMTP 포트"
smtp_user_desc="SMTP 서버 사용자 이름을 입력하십시오."
smtp_user_label="SMTP 사용자 이름"
smtp_pass_desc="SMTP 서버 패스워드를 입력하십시오."
smtp_pass_label="SMTP 패스워드"
smtp_verify_desc="GitLab에 연결되었을 때, SMTP 서버의 SSL 인증서를 검증합니다."
smtp_verify_label="SSL 인증서를 검증하고 안전하지 않은 연결을 거절합니다."
default_gitlab_account_desc="기본 로그인 자격 증명은 root/5iveL!fe입니다"
additional_memory_required="메모리 부족"
recommended_memory="권장 메모리 크기"
db_connect_error="GitLab에 연결하지 못했습니다. 데이터베이스가 제대로 실행 중인지 확인하십시오."
mem_total_check_fail="<a href=\"{2}\" target=\"_blank\">최신 버전의 GitLab</a>을 설치 또는 업데이트하려면 최소 {1} RAM이 필요합니다. Synology NAS의 메모리를 확장하고 다시 시도하십시오."
mem_total_check_warning="GitLab을 설치 또는 업데이트하려면 최소 {1} RAM이 필요합니다. GitLab 실행 시 메모리가 부족하면 예상치 못한 오류가 발생할 수 있으므로, Synology NAS 메모리를 확장하는 것이 좋습니다. <a href=\"{2}\" target=\"_blank\">자세한 정보</a>."
other_used_ports_warning="참고: 이 패키지는 Docker 인터페이스의 {1} 및 {2} 포트도 사용합니다. 방화벽에서 모든 인터페이스에 이러한 포트를 허용하십시오."
domain_value_warning="지정된 도메인 이름도 이메일에서 GitLab 주소로 사용됩니다. GitLab의 도메인 이름이 있으면 "localhost"나 "127.0.0.1" 이외의 이름을 지정하는 것이 좋습니다."
upgrade_warning_page_title="Docker-GitLab 업그레이드"
postgresql_warning_page_desc="Docker-GitLab 새 버전이 백 엔드 데이터베이스로 MariaDB를 <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a>로 교체합니다."
upgrade_agree="데이터베이스를 PostgreSQL로 업그레이드 및 변환에 동의합니다."
intermediate_migrate_warning_page_desc="현재 버전이 오래되어 최신 버전으로 업데이트할 수 없습니다. 최신 버전으로 업데이트하려면 먼저 현재 업데이트 프로세스를 취소하고 <a href=\"{1}\" target=\"_blank\">여기</a>에서 중간 버전을 다운로드하고 해당 버전으로 수동 업데이트한 후 최신 버전으로 업데이트합니다. 수동 업데이트에 자세한 내용은 <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">이 문서</a>를 참조하십시오."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
