#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Velg en metode for å importere data."
wizard_decide_restore="GitLab-databasen eksisterer allerede.<br>Velg en av følgende handlinger:"
wizard_restore="Bruk eksisterende data"
wizard_create_new="Ren installasjon (alle eksisterende data, inkludert konfigurasjonsfiler og databasen, blir fjernet.)"
install_title="Installer GitLab"
install_data_root_desc="Opprett en delt mappe for å lagre dataene for GitLab."
install_data_root_label="Navn på delte mapper"
http_port_desc="Angi HTTP-portnummeret for GitLab."
http_port_label="HTTP-portnummer"
ssh_port_desc="Angi SSH-portnummeret for GitLab."
ssh_port_label="SSH-portnummer"
hostname_label="Domenenavn"
admin_email_desc="Åpne e-postkontoen til GitLab-administrator."
admin_email_label="e-post"
smtp_enable_desc="Aktiver GitLab for å sende e-poster via en SMTP-server."
smtp_enable_label="Aktiver SMTP"
smtp_address_desc="Angi SMTP-serveradressen som verter GitLab."
smtp_address_label="SMTP-adresse"
smtp_port_desc="Angi SMTP-serverporten."
smtp_port_label="SMTP-port"
smtp_user_desc="Angi SMTP-serverbrukernavnet."
smtp_user_label="SMTP-brukernavn"
smtp_pass_desc="Angi SMTP-serverpassordet."
smtp_pass_label="SMTP-passord"
smtp_verify_desc="Når GitLab er tilkoblet, verifiserer du SMTP-serverens SSL-sertifikat."
smtp_verify_label="Verifiser SSL-sertifikatet og avslå usikre tilkoblinger."
default_gitlab_account_desc="Standard innloggingsinformasjon er root/5iveL!fe"
additional_memory_required="Ikke nok minne"
recommended_memory="Anbefalt minnestørrelse"
db_connect_error="Kunne ikke koble til GitLab. Kontroller at databasen kjører som normalt."
mem_total_check_fail="Minimum {1} RAM kreves for å installere eller oppdatere <a href=\"{2}\" target=\"_blank\">den nyeste versjonen av GitLab</a>. Utvid minnet på Synology NAS-enheten din og prøv på nytt."
mem_total_check_warning="Minimum {1} RAM kreves for å installere eller oppdatere GitLab. Utilstrekkelig minne kan føre til uventede feil når du kjører GitLab, så vi anbefaler at du utvider minnet til Synology NAS-enheten din. <a href=\"{2}\" target=\"_blank\">Finn ut mer</a>."
other_used_ports_warning="Merk: Denne pakken bruker også portene {1} og {2} på Docker-grensesnittet. Tillat disse portene for alle grensesnitt i Firewall."
domain_value_warning="Det angitte domenenavnet brukes også som adresse for GitLab i e-poster. Vi anbefaler å angi et annet navn enn «localhost» eller «127.0.0.1» når du har et domenenavn for GitLab."
upgrade_warning_page_title="Oppgrader Docker-GitLab"
postgresql_warning_page_desc="Den nye versjonen av Docker-GitLab erstatter MariaDB med <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> som bakdatabase."
upgrade_agree="Jeg godtar å oppgradere og konvertere databasen til PostgreSQL."
intermediate_migrate_warning_page_desc="Gjeldende versjon er for utdatert til å kunne oppdateres til den nyeste versjonen. Hvis du vil oppdatere til den nyeste versjonen, må du først avbryte den nåværende oppdateringsprosessen, laste ned en midlertidig versjon <a href=\"{1}\" target=\"_blank\">her</a>, oppdatere til denne versjonen manuelt og deretter oppdatere til den nyeste versjonen. Se <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">denne artikkelen</a> for mer informasjon om manuell oppdatering."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
