#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Wybierz metodę importu danych."
wizard_decide_restore="GitLab — baza danych już istnieje.<br>Wybierz dowolną z następujących akcji:"
wizard_restore="Użyj istniejących danych"
wizard_create_new="Czysta instalacja (wszystkie istniejące dane, w tym pliki konfiguracyjne i baza danych, zostaną usunięte)."
install_title="Zainstaluj GitLab"
install_data_root_desc="Utwórz folder współdzielony do przechowywania danych GitLab."
install_data_root_label="Nazwa folderu współdzielonego"
http_port_desc="Wprowadź numer portu HTTP dla GitLab."
http_port_label="Numer portu HTTP"
ssh_port_desc="Wprowadź numer portu SSH dla GitLab."
ssh_port_label="Numer portu SSH"
hostname_label="Nazwa domeny"
admin_email_desc="Wprowadź adres konta e-mail administratora  GitLab."
admin_email_label="e-mail"
smtp_enable_desc="Włącz GitLab, aby wysyłać wiadomości e-mail przez serwer SMTP."
smtp_enable_label="Włącz SMTP"
smtp_address_desc="Wprowadź adres serwera SMTP dla GitLab."
smtp_address_label="Adres SMTP"
smtp_port_desc="Wprowadź port serwera SMTP."
smtp_port_label="Port SMTP"
smtp_user_desc="Wprowadź nazwę użytkownika serwera SMTP."
smtp_user_label="Nazwa użytkownika serwera SMTP"
smtp_pass_desc="Wprowadź hasło serwera SMTP."
smtp_pass_label="Hasło serwera SMTP"
smtp_verify_desc="Po podłączeniu GitLab sprawdź certyfikat SSL serwera SMTP."
smtp_verify_label="Sprawdź certyfikat SSL i odrzuć niezabezpieczone połączenia."
default_gitlab_account_desc="Domyślne poświadczenia logowania to root/5iveL!fe"
additional_memory_required="Niewystarczająca ilość pamięci"
recommended_memory="Zalecana wielkość pamięci"
db_connect_error="Nawiązanie połączenia z GitLab nie powiodło się. Upewnij się, że baza danych działa prawidłowo."
mem_total_check_fail="Minimalny rozmiar pamięci RAM wymagany do zainstalowania lub aktualizacji <a href=\"{2}\" target=\"_blank\">najnowszej wersji programu GitLab</a> to {1}. Rozszerz pamięć w swoim serwerze Synology NAS i spróbuj ponownie."
mem_total_check_warning="Minimalny rozmiar pamięci RAM wymagany do zainstalowania lub aktualizacji programu GitLab to {1}. Niewystarczająca ilość pamięci może spowodować nieoczekiwane błędy podczas podczas pracy programu GitLab, dlatego zaleca się rozszerzenie pamięci serwera Synology NAS. <a href=\"{2}\" target=\"_blank\">Dowiedz się więcej</a>."
other_used_ports_warning="Uwaga: Pakiet ten korzysta również z portów {1} i {2} interfejsu Docker. Udostępnij te porty wszystkim interfejsom w zaporze sieciowej."
domain_value_warning="Określona nazwa domeny będzie również używana jako adres usługi GitLab w wiadomościach e-mail. Zaleca się wprowadzenie nazwy innej niż „localhost” lub „127.0.0.1” dla nazwy domeny GitLab."
upgrade_warning_page_title="Aktualizacja Docker-GitLab"
postgresql_warning_page_desc="Nowa wersja Docker-GitLab zastąpi MariaDB z <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> jako wewnętrzną bazą danych."
upgrade_agree="Zgadzam się na zaktualizowanie i konwersję bazy danych do PostgreSQL."
intermediate_migrate_warning_page_desc="Bieżąca wersja jest przestarzała i nie może zostać zaktualizowana do najnowszej wersji. Aby zaktualizować do najnowszej wersji, najpierw anuluj bieżący proces aktualizacji, pobierz wersję pośrednią <a href=\"{1}\" target=\"_blank\">tutaj</a>, ręcznie zaktualizuj do tej wersji, a następnie aktualizuj do najnowszej wersji. Więcej informacji na temat aktualizacji ręcznej możesz znaleźć w <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">tym artykule</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
