#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Selecione um método para importar dados."
wizard_decide_restore="GitLab a base de dados já existe.<br>Selecione uma das seguintes ações:"
wizard_restore="Utilizar os dados existentes"
wizard_create_new="Limpar a instalação (todos os dados existentes, incluindo os ficheiros de configuração e a base de dados, serão removidos.)"
install_title="Instalar GitLab"
install_data_root_desc="Crie uma pasta partilhada para armazenar os dados de GitLab."
install_data_root_label="Nome da pasta partilhada"
http_port_desc="Introduza o número de porta HTTP para GitLab."
http_port_label="Número de porta HTTP"
ssh_port_desc="Introduza o número de porta SSH para GitLab."
ssh_port_label="Número de porta SSH"
hostname_label="Nome de domínio"
admin_email_desc="Introduza a conta de email de administrador GitLab."
admin_email_label="email"
smtp_enable_desc="Active GitLab para enviar email através de um servidor SMTP."
smtp_enable_label="Activar SMTP"
smtp_address_desc="Introduza o endereço do servidor SMTP que serve de anfitrião a GitLab."
smtp_address_label="Endereço SMTP"
smtp_port_desc="Introduza a porta do servidor SMTP."
smtp_port_label="Porta SMTP"
smtp_user_desc="Introduza o nome de utilizador do servidor SMTP."
smtp_user_label="Nome de utilizador SMTP"
smtp_pass_desc="Introduza a palavra-passe do servidor SMTP."
smtp_pass_label="Palavra-passe SMTP"
smtp_verify_desc="Quando GitLab estiver ligado, verifique o certificado SSL do servidor SMTP."
smtp_verify_label="Verifique o certificado SSL e recuse ligações inseguras."
default_gitlab_account_desc="A credencial predefinida de início de sessão é root/5iveL!fe"
additional_memory_required="Memória insuficiente"
recommended_memory="Tamanho de memória recomendado"
db_connect_error="Falha na ligação a GitLab. Certifique-se de que a base de dados está a funcionar corretamente."
mem_total_check_fail="São necessários, no mínimo, {1} de RAM para instalar ou atualizar <a href=\"{2}\" target=\"_blank\">a versão mais recente do GitLab</a>. Aumente a memória do seu Synology NAS e tente novamente."
mem_total_check_warning="São necessários, no mínimo, {1} de RAM para instalar ou atualizar o GitLab. Uma memória insuficiente pode causar erros inesperados quando executar o GitLab, pelo que se recomenda aumentar a memória do seu Synology NAS. <a href=\"{2}\" target=\"_blank\">Saber mais</a>."
other_used_ports_warning="Nota: Este pacote também usa portas {1} e {2} na interface Docker. Ative estas portas para todas as interfaces na Firewall."
domain_value_warning="O nome de domínio especificado também será utilizado como o endereço do GitLab nos e-mails. Recomendamos que especifique um nome diferente de "localhost" ou "127.0.0.1" quando tiver um nome de domínio para o GitLab."
upgrade_warning_page_title="Atualizar o Docker-GitLab"
postgresql_warning_page_desc="A nova versão do Docker-GitLab vai substituir a MariaDB pela base de dados <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a>."
upgrade_agree="Concordo em atualizar e converter a base de dados para a PostgreSQL."
intermediate_migrate_warning_page_desc="A versão atual está desatualizada para atualizar para a versão mais recente. Para atualizar para a versão mais recente, primeiro cancele o processo de atualização atual, transfira uma versão intermédia <a href=\"{1}\" target=\"_blank\">aqui</a>, atualize manualmente para essa versão e, em seguida, atualize para a versão mais recente. Para obter mais informações sobre atualização manual, consulte <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">este artigo</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
