#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Vänligen välj en metod för att importera data."
wizard_decide_restore="Databasen GitLab finns redan.<br>Vänligen markera någon av följande åtgärder:"
wizard_restore="Använd befintliga data"
wizard_create_new="Ren installation (alla befintliga data, inklusive konfigurationsfiler och databasen tas bort)."
install_title="Installera GitLab"
install_data_root_desc="Skapa en delad mapp för att lagra data för GitLab."
install_data_root_label="Namn på delad mapp"
http_port_desc="Vänligen ange HTTP-portnumret för GitLab."
http_port_label="HTTP-portnummer"
ssh_port_desc="Vänligen ange SSH-portnumret för GitLab."
ssh_port_label="SSH-portnummer"
hostname_label="Domännamn"
admin_email_desc="Vänligen ange GitLab administrator e-postkonto."
admin_email_label="e-post"
smtp_enable_desc="Aktivera GitLab för att skicka e-post via en SMTP-server."
smtp_enable_label="Aktivera SMTP"
smtp_address_desc="Vänligen ange SMTP-serveradressen som är värd för GitLab."
smtp_address_label="SMTP-adress"
smtp_port_desc="Vänligen ange SMTP-serverporten."
smtp_port_label="SMTP-port"
smtp_user_desc="Vänligen ange användarnamn för SMTP-server."
smtp_user_label="Användarnamn för SMTP"
smtp_pass_desc="Vänligen ange SMTP-serverns lösenord."
smtp_pass_label="SMTP-lösenord"
smtp_verify_desc="När GitLab är ansluten, verifiera SMTP-serverns SSL-certifikat."
smtp_verify_label="Verifiera SSL-certifikatet och neka osäkra anslutningar."
default_gitlab_account_desc="Autentiseringsuppgifter är som standard root/5iveL!fe"
additional_memory_required="Otillräckligt minne"
recommended_memory="Rekommenderad minnesstorlek"
db_connect_error="Kunde inte ansluta till GitLab. Vänligen kontrollera att databasen körs ordentligt."
mem_total_check_fail="Det krävs minst {1} RAM för att installera eller uppdatera <a href=\"{2}\" target=\"_blank\">den senaste versionen av GitLab</a>. Vänligen utöka minnet på din Synology NAS och försök igen."
mem_total_check_warning="Det krävs minst {1} RAM för att installera eller uppdatera GitLab. Otillräckligt med minne kan orsaka oväntade fel när du kör GitLab och du rekommenderas därför att utöka minnet på din Synology NAS. <a href=\"{2}\" target=\"_blank\">Få reda på mer</a>."
other_used_ports_warning="Obs! I paketet används också portarna {1} och {2} på Dockers gränssnitt. Tillåt dessa portar för alla gränssnitt i brandväggen."
domain_value_warning="Det angivna domännamnet används också som GitLab-adress i e-postmeddelanden. Vi rekommenderar att du anger ett annat namn än "localhost" eller "127.0.0.1" när du har ett domännamn för GitLab."
upgrade_warning_page_title="Uppgradera Docker-GitLab"
postgresql_warning_page_desc="Den nya versionen av Docker-GitLab ersätter MariaDB med <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> som databas på serversidan."
upgrade_agree="Jag samtycker till att uppgradera och konvertera databasen till PostgreSQL."
intermediate_migrate_warning_page_desc="Den aktuella versionen är för gammal för uppdatering till den senaste versionen. Om du vill uppdatera till den senaste versionen avbryter du först den aktuella uppdateringsprocessen, hämtar en mellanversion <a href=\"{1}\" target=\"_blank\">här</a>, uppdaterar manuellt till den versionen och uppdaterar sedan till den senaste versionen. Mer information om manuell uppdatering finns i <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">den här artikeln</a>."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
