#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Please select a method to import data."
wizard_decide_restore="GitLab database already exists.<br>Please select either of the following actions:"
wizard_restore="Use existing data"
wizard_create_new="Clean install (All existing data, including configuration files and the database, will be removed.)"
install_title="Install GitLab"
install_data_root_desc="Create a shared folder to store the data of GitLab."
install_data_root_label="Shared folder name"
http_port_desc="Please enter the HTTP port number for GitLab."
http_port_label="HTTP port number"
ssh_port_desc="Please enter the SSH port number for GitLab."
ssh_port_label="SSH port number"
hostname_label="Domain name"
admin_email_desc="Please enter the GitLab administrator email account."
admin_email_label="email"
smtp_enable_desc="Enable GitLab to send emails through a SMTP server."
smtp_enable_label="Enable SMTP"
smtp_address_desc="Please enter the SMTP server address hosting GitLab."
smtp_address_label="SMTP address"
smtp_port_desc="Please enter the SMTP server port."
smtp_port_label="SMTP port"
smtp_user_desc="Please enter the SMTP server username."
smtp_user_label="SMTP username"
smtp_pass_desc="Please enter the SMTP server password."
smtp_pass_label="SMTP password"
smtp_verify_desc="When GitLab is connected, verify the SMTP server's SSL certificate."
smtp_verify_label="Verify the SSL certificate and refuse unsafe connections."
default_gitlab_account_desc="The default login credential is root/5iveL!fe"
additional_memory_required="หน่วยความจำไม่เพียงพอ"
recommended_memory="ขนาดหน่วยความจำที่แนะนำ"
db_connect_error="เชื่อมต่อกับ GitLabไม่สำเร็จ โปรดตรวจสอบว่าฐานข้อมูลทำงานอย่างถูกต้อง"
mem_total_check_fail="ต้องการ RAM อย่างน้อย {1} เพื่อติดตั้งหรืออัปเดต<a href=\"{2}\" target=\"_blank\"> GitLab เวอร์ชันล่าสุด</a> โปรดขยายหน่วยความจำของ Synology NAS ของคุณและลองอีกครั้ง"
mem_total_check_warning="ต้องการ RAM อย่างน้อย {1} เพื่อติดตั้งหรืออัปเดต GitLab หน่วยความจำไม่เพียงพออาจทำให้เกิดข้อผิดพลาดที่ไม่คาดหมายเมื่อคุณรัน GitLab ดังนั้นเราขอแนะนำให้คุณขยายหน่วยความจำของ Synology NAS ของคุณ <a href=\"{2}\" target=\"_blank\">เรียนรู้เพิ่มเติม</a>"
other_used_ports_warning="หมายเหตุ: แพคเกจนี้ยังใช้พอร์ต {1} และ {2} บนอินเตอร์เฟซ Docker โปรดอนุญาตพอร์ตเหล่านี้สำหรับอินเตอร์เฟซทั้งหมดในไฟร์วอลล์"
domain_value_warning="จะใช้ชื่อโดเมนที่ระบุเป็นแอดเดรสของ GitLab ในอีเมลด้วย เราขอแนะนำให้ระบุชื่ออื่นที่ไม่ใช่ "localhost" หรือ "127.0.0.1" เมื่อคุณตั้งชื่อโดเมนสำหรับ GitLab"
upgrade_warning_page_title="อัปเกรด Docker-GitLab"
postgresql_warning_page_desc="Docker-GitLab เวอร์ชันใหม่จะแทนที่ MariaDB ด้วย <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> เป็นฐานข้อมูลแบ็กเอนด์"
upgrade_agree="ฉันตกลงที่จะอัปเกรดและแปลงฐานข้อมูลเป็น PostgreSQL"
intermediate_migrate_warning_page_desc="เวอร์ชันปัจจุบันล้าสมัยเกินกว่าจะอัปเดตเป็นเวอร์ชันล่าสุด หากต้องการอัปเดตเป็นเวอร์ชันล่าสุด ก่อนอื่นให้ยกเลิกกระบวนการอัปเดตที่ทำอยู่ แล้วดาวน์โหลดเวอร์ชันกลางจาก<a href=\"{1}\" target=\"_blank\">ที่นี่</a> อัปเดตด้วยตนเองเป็นเวอร์ชันนั้น และจากนั้นอัปเดตเป็นเวอร์ชันล่าสุด สำหรับข้อมูลเพิ่มเติมเกี่ยวกับการอัปเดตด้วยตนเอง โปรดดู<a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">บทความนี้</a>"
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
