#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"
GITLAB_INTERMEDIATE_VERSION="0055"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_upgrade_to_intermediate_version_first()
{
	if [ ! -f "/usr/syno/etc/packages/Docker-GitLab/config" ]; then
		return 1
	fi

	version=$(get_key_value "/usr/syno/etc/packages/Docker-GitLab/config" "VERSION")

	if [ -z "$version" ] || [ "$version" -lt "$GITLAB_INTERMEDIATE_VERSION" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Lütfen verileri içeri aktarmak için bir yöntem seçin."
wizard_decide_restore="GitLab veritabanı zaten mevcut.<br>Lütfen aşağıdaki eylemlerden birisini seçin:"
wizard_restore="Mevcut verileri kullan"
wizard_create_new="Kurulumu kaldır (Tüm mevcut veriler, yapılandırma dosyaları ve veritabanı dahil, kaldırılacaktır.)"
install_title="GitLab öğesini kur"
install_data_root_desc="GitLab verisini saklamak için bir paylaşımlı klasör oluşturun."
install_data_root_label="Paylaşımlı klasör adı"
http_port_desc="Lütfen GitLab için HTTP bağlantı noktası numarasını girin."
http_port_label="HTTP bağlantı noktası numarası"
ssh_port_desc="Lütfen GitLab için SSH bağlantı noktası numarasını girin."
ssh_port_label="SSH bağlantı noktası numarası"
hostname_label="Etki alanı ismi"
admin_email_desc="Lütfen GitLab administrator e-posta adresini girin."
admin_email_label="e-posta"
smtp_enable_desc="Bir SMTP sunucusu yoluyla e-posta göndermek için GitLab'ı etkinleştirin."
smtp_enable_label="SMTP etkinleştir"
smtp_address_desc="GitLab barındıran SMTP sunucu adresini girin."
smtp_address_label="SMTP adresi"
smtp_port_desc="Lütfen SMTP sunucusu bağlantı noktasını girin."
smtp_port_label="SMTP bağlantı noktası"
smtp_user_desc="Lütfen SMTP sunucusu kullanıcı adını girin."
smtp_user_label="SMTP kullanıcı adı"
smtp_pass_desc="Lütfen SMTP sunucusu parolasını girin."
smtp_pass_label="SMTP parolası"
smtp_verify_desc="GitLab bağlandığında, SMTP sunucusunun SSL sertifikasını doğrulayın."
smtp_verify_label="SSL sertifikasını doğrulayın ve güvensiz bağlantıları reddedin."
default_gitlab_account_desc="Varsayılan oturum açma bilgisi root/5iveL!fe"
additional_memory_required="Yetersiz hafıza"
recommended_memory="Önerilen bellek boyutu"
db_connect_error="GitLab öğesine bağlanılamadı. Veritabanının uygun şekilde çalıştığından emin olun."
mem_total_check_fail="Asgari {1} RAM, <a href=\"{2}\" target=\"_blank\">GitLab'ın en son sürümünü</a> kurmak veya bu sürüme güncellemek için gereklidir. Lütfen Synology NAS'ınızın belleğini artırın ve tekrar deneyin."
mem_total_check_warning="GitLab kurmak veya güncellemek için asgari {1} RAM gereklidir. Yetersiz bellek GitLab'ı çalıştırırken beklenmedik hatalara neden olabilir, bu nedenle Synology NAS'ınızın belleğini artırmanız önerilir. <a href=\"{2}\" target=\"_blank\">Daha fazla bilgi</a>."
other_used_ports_warning="Not: Bu paket Docker arayüzünde {1} ve {2} bağlantı noktalarını da kullanır. Lütfen Güvenlik Duvarı'na giderek tüm arayüzler için bu bağlantı noktalarına izin verin."
domain_value_warning="Belirtilen etki alanı adı, e-postalarda GitLab adresi olarak da kullanılır. GitLab için bir etki alanı adınız varsa "localhost" veya "127.0.0.1" dışında bir ad belirtmenizi öneririz."
upgrade_warning_page_title="Docker-GitLab'ı yükselt"
postgresql_warning_page_desc="Docker-GitLab'ın yeni sürümü MariaDB'nin <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a>ile arka uç veritabanı olarak yerini alacak."
upgrade_agree="Veritabanını PostgreSQL'e yükseltmeyi ve dönüştürmeyi kabul ediyorum."
intermediate_migrate_warning_page_desc="Mevcut sürüm, en yeni sürüme güncellemek için çok eskidir. En yeni sürüme güncellemek için ilk önce mevcut güncelleme sürecini iptal edin, <a href=\"{1}\" target=\"_blank\">buradan</a> ara bir sürüm indirin, o sürüme manuel olarak güncelleyin ve sonra en yeni sürüme güncelleyin. Manuel güncelleme hakkında daha fazla bilgi için lütfen <a href=\"https://www.synology.com/knowledgebase/DSM/tutorial/Service_Application/How_to_install_applications_with_Package_Center#t2.3\" target=\"_blank\">bu makaleye</a> bakın."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v12.10.6/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		memtitle="$recommended_memory"
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		memtitle="$additional_memory_required"
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"step_title": "$memtitle",
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
PageGitlabIntermediateMigration()
{
	intermediate_migrate_warning_page_desc=${intermediate_migrate_warning_page_desc//\{1\}/"https://sy.to/sqso1"}
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "<b>$(echo "$intermediate_migrate_warning_page_desc" | quote_json)</b>",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return false;}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""
	local gitlab_migrate_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	if need_upgrade_to_intermediate_version_first; then
		gitlab_migrate_page="$(PageGitlabIntermediateMigration)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	upgrade_page=$(page_append "$upgrade_page" "$gitlab_migrate_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
